using System;
using System.Collections.Generic;
using System.Text;
using System.Data.Common;
using System.Data;

namespace Mesware.MesControl.DataProvider
{

	/// <summary>
	/// Represents a DataObjects .NET transaction to be made in a MEScontrol.
	/// </summary>
	public class MEScontrolTransaction : DbTransaction
	{
		private MEScontrolConnection connection;
		private IsolationLevel level;

		/// <summary>
		/// Specifies the <see cref="DbConnection"/> object associated with the transaction.
		/// </summary>
		protected override DbConnection DbConnection {
			get { return connection; }
		}

		/// <summary>
		/// Specifies the <see cref="MEScontrolConnection"/> object associated with the transaction.
		/// </summary>
		public new MEScontrolConnection Connection {
			get { return connection; }
		}

		/// <summary>
		/// Specifies the <see cref="IsolationLevel"/> for this transaction.
		/// </summary>
		public override IsolationLevel IsolationLevel {
			get { return level; }
		}

		/// <summary>
		/// Commits the database transaction.
		/// </summary>
		public override void Commit()
		{
		}

		/// <summary>
		/// Rolls back a transaction from a pending state.
		/// </summary>
		public override void Rollback()
		{
		}

		#region IDisposable Members
		void IDisposable.Dispose()
		{
			transaction = null;
		}
		#endregion

		internal MEScontrolTransaction(MEScontrolConnection connection, IsolationLevel level)
		{
			this.level = level;
			this.connection = connection;
		}
	}
}
