using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;
using System.Reflection;
using Type=System.Type;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Represents source of data for <see cref="MEScontrolDataReader"/>
	/// based on <see cref="DataService"/>.
	/// </summary>
	public class ServiceDataReaderSource : MethodDataReaderSource
	{
		protected override void ProcessCommand(
			string commandText,
			out string newCommandText,
			out object targetObject,
			out string methodName,
			out string[] parameters)
		{
			Regex reges = new Regex(@"\s*((?<Namespace>.*)\.)?(?<ClassName>[^\.\(]*)\.(?<MethodName>[^\.\(]*)\((?'Arguments'.*)\)\s*$");
			Match match = reges.Match(commandText);
			if (!match.Success)
				throw new InvalidOperationException(TranslationResources.CommandTextIsInvalid);
			string nameSpace = match.Groups["Namespace"].Value;
			string className = match.Groups["ClassName"].Value;
			string arguments = match.Groups["Arguments"].Value;
			string fullClassName = nameSpace+"."+className;
			targetObject = null;

			if (nameSpace==null || nameSpace=="")
				fullClassName = className;
			int cnt = 0;
			// TODO: here you need to implement search of type wich contains method provides data
			/*foreach (Service s in session.Services) {
				if (s.ShortName==fullClassName || s.SourceType.FullName==fullClassName && s.ServiceType==DataServiceType.Shared) {
					targetObject = ServiceFactory.GetService(session, s.SourceType);
					cnt++;
				}
			}*/
			if (targetObject==null)
				throw new InvalidOperationException(string.Format(TranslationResources.CanNotFindServiceWithName, fullClassName));
			if (cnt>1)
				throw new InvalidOperationException(string.Format(TranslationResources.ThereIsMoreThanOneServiceWithName, fullClassName));

			methodName =  match.Groups["MethodName"].Value;
			newCommandText = commandText.Replace(fullClassName+".", "");
			if (!string.IsNullOrEmpty(arguments.Trim()))
				parameters = arguments.Split(',');
			else
				parameters = new string[0]; 

		}

		/// <summary>
		/// Initializes a new instance of the <see cref="QueryDataReaderSource"/> class.
		/// </summary>
		/// <param name="session">Current <see cref="Session"/>.</param> 
		/// <param name="commandText">Command text of the source.</param>
		public ServiceDataReaderSource(string commandText)
			: base(commandText)
		{
		}
	}
}
