using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;
using System.Reflection;
using Type=System.Type;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Represents source of data for <see cref="MEScontrolDataReader"/>
	/// based on specified object.
	/// </summary>
	public class SpecifiedObjectDataReaderSource : MethodDataReaderSource
	{
		private object targetObject;

		protected override void ProcessCommand(
			string commandText,
			out string newCommandText,
			out object targetObject,
			out string methodName,
			out string[] parameters)
		{
			Regex reges = new Regex(@"\s*(?<MethodName>[^\.\(]*)\((?'Arguments'.*)\)\s*$");
			Match match = reges.Match(commandText);
			if (!match.Success)
				throw new InvalidOperationException(TranslationResources.CommandTextIsInvalid);			
			string arguments = match.Groups["Arguments"].Value;

			targetObject = this.targetObject;

			methodName =  match.Groups["MethodName"].Value;			
			if (!string.IsNullOrEmpty(arguments.Trim()))
				parameters = arguments.Split(',');
			else
				parameters = new string[0];
			newCommandText = commandText; 

		}

		/// <summary>
		/// Initializes a new instance of the <see cref="QueryDataReaderSource"/> class.
		/// </summary>
		/// <param name="commandText">Command text of the source.</param>
		public SpecifiedObjectDataReaderSource(string commandText, object targetObject)
			: base(commandText)
		{
			this.targetObject = targetObject;
		}
	}
}
