using System;
using System.Collections.Generic;
using System.Text;
using Stimulsoft.Report.Dictionary;
using System.Data;
using Stimulsoft.Report;
using log4net;
using Mesware.MesControl.DataProvider.Properties;
using System.Windows.Forms;
using System.IO;
using System.Globalization;
using Stimulsoft.Base.Localization;
using System.Threading;
using System.CodeDom.Compiler;

namespace Mesware.MesControl.DataProvider
{
	public class MEScontrolDataAdapterService : StiSqlAdapterService
	{

		public override string ServiceName {
			get { return TranslationResources.DataFromMEScontrolConnection; }
		}

		#region StiDataAdapterService override
		public override StiDataColumnsCollection GetColumnsFromData(StiData data, StiDataSource dataSource)
		{
			StiDataColumnsCollection dataColumns = new StiDataColumnsCollection();
			StiSqlSource sqlSource = dataSource as StiSqlSource;

			if (sqlSource.SqlCommand!=null && sqlSource.SqlCommand.Length>0) {
				if (data.Data is MEScontrolConnection) {
 
					MEScontrolConnection connection = (MEScontrolConnection)data.Data;
					OpenConnection(connection, data, dataSource.Dictionary);
					// tricky way
					string[] parts = sqlSource.SqlCommand.Split('(');
					string partBeforeBracket = parts[0];
					if (parts.Length>1 && !partBeforeBracket.Contains(".") && dataSource.Dictionary.Report.CompiledReport==null) {
						dataSource.Dictionary.Report.Compile();
						if (dataSource.Dictionary.Report.CompilerResults.Errors.HasErrors) {
							StringBuilder errorMessage = new StringBuilder();
							foreach (CompilerError error in dataSource.Dictionary.Report.CompilerResults.Errors)
								if (!error.IsWarning)
									errorMessage.AppendLine(error.ToString());
							throw new InvalidOperationException(errorMessage.ToString()); 
						}
					}

					using (MEScontrolDataAdapter dataAdapter = new MEScontrolDataAdapter(sqlSource.SqlCommand, connection, dataSource.Dictionary.Report.CompiledReport)) {
						DataTable dataTable = new DataTable();
						dataTable.TableName = sqlSource.Name;
						dataAdapter.SelectCommand.Prepare();
						dataAdapter.FillSchema(dataTable, SchemaType.Source);
						foreach (DataColumn column in dataTable.Columns)
							dataColumns.Add(new StiDataColumn(column.ColumnName, column.Caption, column.DataType));
						dataTable.Dispose();
					}
					CloseConnection(data, connection);
				}
			}
			return dataColumns;
		}

		public override void SetDataSourceNames(StiData data, StiDataSource dataSource)
		{
			base.SetDataSourceNames(data, dataSource);

			dataSource.Name = "MEScontrolSource";
			dataSource.Alias = "MEScontrolSource";
		}

		public override Type GetDataSourceType()
		{
			return typeof(MEScontrolSource);
		}

		public override Type[] GetDataTypes()
		{
			return new Type[] { typeof(MEScontrolConnection) };
		}

		protected override void OpenConnection(IDbConnection connection, StiData data, StiDictionary dictionary)
		{
			MEScontrolConnection mesConnection = connection as MEScontrolConnection;
			if (mesConnection==null)
				base.OpenConnection(connection, data, dictionary);
			else {
				if (mesConnection.State!=ConnectionState.Closed)
					return; 
				mesConnection.Open();
			}
		}

		public override void ConnectDataSourceToData(StiDictionary dictionary, StiDataSource dataSource, bool loadData)
		{
			dataSource.Disconnect();
			if (!loadData) {
				dataSource.DataTable = new DataTable();
				return;
			}

			StiSqlSource sqlSource = (StiSqlSource)dataSource;

			foreach (StiData data in dataSource.Dictionary.DataStore) {
				if (data.Name==sqlSource.NameInSource) {
					if (data.Data is MEScontrolConnection) {
						MEScontrolConnection connection = data.ViewData as MEScontrolConnection;						
						OpenConnection(connection, data, dataSource.Dictionary);


						MEScontrolDataAdapter adapter = new MEScontrolDataAdapter(sqlSource.SqlCommand, connection, dataSource.Dictionary.Report);
						sqlSource.DataAdapter = adapter;
						MEScontrolCommand cmd = (MEScontrolCommand)adapter.SelectCommand;

						foreach (StiDataParameter parameter in sqlSource.Parameters)
							cmd.Parameters.Add(new MEScontrolParameter(parameter.Name, (DbType)parameter.Type));

						DataTable dataTable = new DataTable();
						dataTable.TableName = sqlSource.Name;
						dataSource.DataTable = dataTable;
						
						if (loadData) { 
							
							List<string> expressions = new List<string>();
							foreach (StiDataColumn column in sqlSource.Columns)
								if (!(column is StiCalcDataColumn))
									cmd.AddField(column.NameInSource, column.Type);
							
							if (sqlSource.Parameters.Count>0) {	
								sqlSource.UpdateParameters();
							}
							else {
								adapter.Fill(dataTable);
								sqlSource.CheckColumnsIndexs();
							}
						}
						else 
							((MEScontrolDataAdapter)sqlSource.DataAdapter).FillSchema(dataTable, SchemaType.Source);

						break;
					}
				}
			}
		}
		#endregion

		public override void CreateConnectionInDataStore(StiDictionary dictionary, StiSqlDatabase database)
		{
			#region remove all old data from datastore
			int index = 0;
			foreach (StiData oldData in dictionary.DataStore) {
				if (oldData.Name==database.Name) {
					dictionary.DataStore.RemoveAt(index);
					break;
				}
				index++;
			}
			#endregion

			MEScontrolConnection sqlConnection = new MEScontrolConnection(database.ConnectionString);

			StiData data = new StiData(database.Name, sqlConnection);
			data.IsReportData = true;
			dictionary.DataStore.Add(data);
		}

		public override string TestConnection(string connectionString)
		{
			try {
				using (MEScontrolConnection sqlConnection = new MEScontrolConnection(connectionString)) {
					sqlConnection.Open();
					sqlConnection.Close();
				}
				return TranslationResources.TestOK;
			}
			catch (Exception ex) {
				return ex.Message; 
			}
		}

		public static void Init()
		{
			StiConfig.Services.Add(new MEScontrolDataAdapterService());
			StiConfig.Services.Add(new MEScontrolDatabase());
		}
	}
}
