using System;
using System.Collections.Generic;
using System.Text;
using Stimulsoft.Report.Dictionary;
using Mesware.MesControl.Model;

namespace Mesware.MesControl.DataProvider
{
	public class SumDimensionalValueFunctionService : StiAggregateFunctionService
	{
		#region StiService override
		/// <summary>
		/// Gets a service name.
		/// </summary>
		public override string ServiceName
		{
			get
			{
				return "SumDV";
			}
		}
		#endregion

		#region IStiStateSaveRestore
		/// <summary>
		/// Saves the current state of an object.
		/// </summary>
		/// <param name="stateName">A name of the state being saved.</param>
		public override void SaveState(string stateName)
		{
		}


		/// <summary>
		/// Restores the earlier saved object state.
		/// </summary>
		/// <param name="stateName">A name of the state being restored.</param>
		public override void RestoreState(string stateName)
		{
		}
		#endregion

		#region Fields
		private DimensionalValue summary;
		private bool first;
		#endregion

		#region Methods
		/// <summary>
		/// First initialization.
		/// </summary>
		public override void Init()
		{
			if ((!this.RunningTotal) || this.IsFirstInit)
			{
				first = true;
			}
		}


		/// <summary>
		/// A value calculation.
		/// </summary>
		/// <param name="value">Value.</param>
		public override void CalcItem(object value)
		{
			if (value==null || value==DBNull.Value) return;
			if (first)
				summary = (DimensionalValue)value;
			else
				summary += (DimensionalValue)value;
			first = false;
		}


		/// <summary>
		/// Returns the calculation result.
		/// </summary>
		/// <returns>Calculation result.</returns>
		public override object GetValue()
		{
			if (first==true)
				return new DimensionalValue("", 0);
			return summary;
		}

		public override void SetValue(object value)
		{
			this.summary = (DimensionalValue)value;
		}


		/// <summary>
		/// Returns the type of the result.
		/// </summary>
		public override Type GetResultType()
		{
			return typeof(DimensionalValue);
		}

		#endregion

		#region Properties
		/// <summary>
		/// Gets value, indicates that this function require a parameter.
		/// </summary>
		public override bool RecureParam
		{
			get
			{
				return true;
			}
		}
		#endregion
	}
}
